package com.sun.org.apache.bcel.internal.generic;

import com.sun.org.apache.bcel.internal.util.ByteSequence;
import com.sun.org.apache.bcel.internal.classfile.Constant;
import com.sun.org.apache.bcel.internal.Const;
import com.sun.org.apache.bcel.internal.classfile.ConstantClass;
import com.sun.org.apache.bcel.internal.classfile.ConstantPool;
import java.io.IOException;
import java.io.DataOutputStream;

public abstract class CPInstruction extends Instruction implements TypedInstruction, IndexedInstruction
{
    private int index;
    
    CPInstruction() {
    }
    
    protected CPInstruction(short opcode, int index) {
        super(opcode, (short)3);
        this.setIndex(index);
    }
    
    @Override
    public void dump(DataOutputStream out) throws IOException {
        out.writeByte((int)super.getOpcode());
        out.writeShort(this.index);
    }
    
    @Override
    public String toString(boolean verbose) {
        return super.toString(verbose) + " " + this.index;
    }
    
    @Override
    public String toString(ConstantPool cp) {
        Constant c = cp.getConstant(this.index);
        String str = cp.constantToString(c);
        if ((c instanceof ConstantClass)) {
            str = str.replace('.', '/');
        }
        return Const.getOpcodeName((int)super.getOpcode()) + " " + str;
    }
    
    @Override
    protected void initFromFile(ByteSequence bytes, boolean wide) throws IOException {
        this.setIndex(bytes.readUnsignedShort());
        super.setLength(3);
    }
    
    @Override
    public final int getIndex() {
        return this.index;
    }
    
    @Override
    public void setIndex(int index) {
        if (index < 0) {
            throw new ClassGenException("Negative index value: " + index);
        }
        this.index = index;
        return;
    }
    
    @Override
    public Type getType(ConstantPoolGen cpg) {
        ConstantPool cp = cpg.getConstantPool();
        String name = cp.getConstantString(this.index, (byte)7);
        if (!name.startsWith("[")) {
            name = "L" + name + ";";
        }
        return Type.getType(name);
    }
}
